"use strict";
var application = require("application");
var common = require("./carousel-view-common");
var VIEWS_STATES = "_viewStates";
var CarouselView = (function (_super) {
    __extends(CarouselView, _super);
    function CarouselView() {
        return _super.apply(this, arguments) || this;
    }
    Object.defineProperty(CarouselView.prototype, "android", {
        get: function () {
            return this._android;
        },
        enumerable: true,
        configurable: true
    });
    CarouselView.prototype._createUI = function () {
        this._android = new android.widget.RelativeLayout(application.android.currentContext);
    };
    CarouselView.prototype.onLoaded = function () {
        if (this.orientation == 0)
            this._viewPager = new android.support.v4.view.ViewPager(this._context);
        else {
            ensureVerticalViewPagerClass();
            this._viewPager = new VerticalViewPagerClass(application.android.currentContext);
        }
        // TODO: Create a property for the user set the margin color (to avoid seeing a view in the background)
        this._viewPager.setBackgroundColor(android.graphics.Color.parseColor("#FFFFFFFF"));
        var res = android.content.res.Resources;
        var margin = this.interPageSpacing * res.getSystem().getDisplayMetrics().density;
        this._viewPager.setPageMargin(margin);
        var that = new WeakRef(this);
        ensurePagerAdapterClass();
        this._viewPager.setAdapter(new PagerAdapterClass(this));
        var layoutParams = new android.widget.RelativeLayout.LayoutParams(android.widget.RelativeLayout.LayoutParams.MATCH_PARENT, android.widget.RelativeLayout.LayoutParams.MATCH_PARENT);
        this._android.addView(this._viewPager, layoutParams);
        var indicators;
        if (this.showIndicators) {
            ensureCirclePageIndicatorClass();
            indicators = new CirclePageIndicatorClass(application.android.currentContext);
            if (this.orientation == 0) {
                layoutParams = new android.widget.RelativeLayout.LayoutParams(android.widget.RelativeLayout.LayoutParams.MATCH_PARENT, android.widget.RelativeLayout.LayoutParams.WRAP_CONTENT);
                layoutParams.addRule(android.widget.RelativeLayout.ALIGN_PARENT_BOTTOM);
            }
            else {
                indicators.mOrientation = 1;
                layoutParams = new android.widget.RelativeLayout.LayoutParams(android.widget.RelativeLayout.LayoutParams.WRAP_CONTENT, android.widget.RelativeLayout.LayoutParams.MATCH_PARENT);
                layoutParams.addRule(android.widget.RelativeLayout.ALIGN_PARENT_RIGHT);
            }
            indicators.SetViewPager(this._viewPager, this.position);
            indicators.mSnapPage = this.position;
            this._android.addView(indicators, layoutParams);
        }
        ensurePageChangedListenerClass();
        this._viewPager.setOnPageChangeListener(new PageChangedListenerClass(this, indicators));
        this._viewPager.setCurrentItem(this.position, false);
        var eventData = {
            eventName: "positionSelected",
            object: this
        };
        this.notify(eventData);
    };
    CarouselView.prototype.insertPage = function (position, bindingContext) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this._viewPager != null))
                            return [3 /*break*/, 2];
                        if (position > this.itemsSource.length)
                            throw new Error("Index out of bounds (position > itemsSource length).");
                        if (position < 0)
                            throw new Error("Index out of bounds (position < 0).");
                        if (position == this.itemsSource.length)
                            this.itemsSource.push(bindingContext);
                        else
                            this.itemsSource.splice(position, 0, bindingContext);
                        if (position > 1)
                            this._viewPager.getAdapter().notifyDataSetChanged();
                        else
                            this._viewPager.setAdapter(new PagerAdapterClass(this));
                        return [4 /*yield*/, this.delay(100)];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    CarouselView.prototype.removePage = function (position) {
        return __awaiter(this, void 0, void 0, function () {
            var newPos;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this._viewPager != null))
                            return [3 /*break*/, 6];
                        if (position > this.itemsSource.length - 1)
                            throw new Error("Index out of bounds (position > itemsSource length - 1).");
                        if (position < 0)
                            throw new Error("Index out of bounds (position < 0).");
                        if (!(position == this.position))
                            return [3 /*break*/, 5];
                        newPos = position - 1;
                        if (newPos == -1)
                            newPos = 0;
                        if (!(position == 0))
                            return [3 /*break*/, 2];
                        this._viewPager.setCurrentItem(1, true);
                        return [4 /*yield*/, this.delay(100)];
                    case 1:
                        _a.sent();
                        this.itemsSource.splice(position, 1);
                        //this._viewPager.setAdapter(new PagerAdapterClass(this));
                        this._viewPager.getAdapter().notifyDataSetChanged();
                        this._viewPager.setCurrentItem(0, false);
                        this.position = 0;
                        return [3 /*break*/, 4];
                    case 2:
                        this._viewPager.setCurrentItem(newPos, true);
                        return [4 /*yield*/, this.delay(100)];
                    case 3:
                        _a.sent();
                        this.itemsSource.splice(position, 1);
                        if (position == 1)
                            this._viewPager.setAdapter(new PagerAdapterClass(this));
                        else
                            this._viewPager.getAdapter().notifyDataSetChanged();
                        this.position = newPos;
                        _a.label = 4;
                    case 4: return [3 /*break*/, 6];
                    case 5:
                        this.itemsSource.splice(position, 1);
                        if (position == 1)
                            this._viewPager.setAdapter(new PagerAdapterClass(this));
                        else
                            this._viewPager.getAdapter().notifyDataSetChanged();
                        _a.label = 6;
                    case 6: return [2 /*return*/];
                }
            });
        });
    };
    CarouselView.prototype.setCurrentPage = function (position) {
        if (this._viewPager != null) {
            if (position > this.itemsSource.length - 1)
                throw new Error("Index out of bounds (position > itemsSource length - 1).");
            if (position < 0)
                throw new Error("Index out of bounds (position < 0).");
            this.position = position;
            this._viewPager.setCurrentItem(position, true);
        }
    };
    CarouselView.prototype.itemsSourceChanged = function () {
        if (this._viewPager != null) {
            if (this.position > this.itemsSource.length - 1)
                this.position = this.itemsSource.length - 1;
            ensurePagerAdapterClass();
            this._viewPager.setAdapter(new PagerAdapterClass(this));
            this._viewPager.setCurrentItem(this.position, false);
            var eventData = {
                eventName: "positionSelected",
                object: this
            };
            this.notify(eventData);
        }
    };
    CarouselView.prototype.delay = function (ms) {
        return new Promise(function (resolve) { return setTimeout(resolve, ms); });
    };
    return CarouselView;
}(common.CarouselView));
exports.CarouselView = CarouselView;
var PagerAdapterClass;
function ensurePagerAdapterClass() {
    if (PagerAdapterClass) {
        return;
    }
    var PagerAdapterInner = (function (_super) {
        __extends(PagerAdapterInner, _super);
        function PagerAdapterInner(_owner) {
            var _this = _super.call(this) || this;
            _this._owner = _owner;
            return global.__native(_this);
        }
        PagerAdapterInner.prototype.getCount = function () {
            // FIX: populate the carousel with data after being loaded in UI
            if (this._owner.itemsSource == null)
                return 0;
            return this._owner.itemsSource.length;
        };
        PagerAdapterInner.prototype.isViewFromObject = function (view, objectValue) {
            return view === objectValue;
        };
        PagerAdapterInner.prototype.instantiateItem = function (container, position) {
            // FIX: populate the carousel with data after being loaded in UI
            var item;
            if (this._owner.itemsSource != null)
                item = this._owner.itemsSource.getItem(position);
            var view = this._owner.templateSelector.OnSelectTemplate(position, item);
            var obj = view;
            //obj._onAttached(application.android.currentContext);
            this._owner._addView(obj);
            obj.android.tag = position;
            if (this[VIEWS_STATES]) {
                obj.android.restoreHierarchyState(this[VIEWS_STATES]);
            }
            container.addView(obj.android);
            obj.onLoaded();
            return obj.android;
        };
        PagerAdapterInner.prototype.destroyItem = function (container, position, objectValue) {
            var pager = container;
            pager.removeView(objectValue);
        };
        PagerAdapterInner.prototype.getItemPosition = function (objectValue) {
            var tag = Number(objectValue.getTag());
            var position = this._owner.position;
            if (tag == position)
                return tag;
            return android.support.v4.view.PagerAdapter.POSITION_NONE;
        };
        PagerAdapterInner.prototype.saveState = function () {
            if (!this[VIEWS_STATES]) {
                this[VIEWS_STATES] = new android.util.SparseArray();
            }
            var mViewStates = this[VIEWS_STATES];
            var mViewPager = this._owner.android;
            var count = mViewPager.getChildCount();
            for (var i = 0; i < count; i++) {
                var c = mViewPager.getChildAt(i);
                if (c.isSaveFromParentEnabled()) {
                    c.saveHierarchyState(mViewStates);
                }
            }
            var bundle = new android.os.Bundle();
            bundle.putSparseParcelableArray(VIEWS_STATES, mViewStates);
            return bundle;
        };
        PagerAdapterInner.prototype.restoreState = function (state, loader) {
            var bundle = state;
            bundle.setClassLoader(loader);
            this[VIEWS_STATES] = bundle.getSparseParcelableArray(VIEWS_STATES);
        };
        return PagerAdapterInner;
    }(android.support.v4.view.PagerAdapter));
    PagerAdapterClass = PagerAdapterInner;
}
var PageChangedListenerClass;
function ensurePageChangedListenerClass() {
    if (PageChangedListenerClass) {
        return;
    }
    var PageChangedListenerInner = (function (_super) {
        __extends(PageChangedListenerInner, _super);
        function PageChangedListenerInner(owner, indicators) {
            var _this = _super.call(this) || this;
            _this._owner = owner;
            _this._indicators = indicators;
            return global.__native(_this);
        }
        PageChangedListenerInner.prototype.onPageScrolled = function (position, positionOffset, positionOffsetPixels) {
            if (this._indicators != null) {
                this._indicators.mCurrentPage = position;
                this._indicators.mCurrentOffset = positionOffsetPixels;
                this._indicators.UpdatePageSize();
                this._indicators.invalidate();
            }
        };
        PageChangedListenerInner.prototype.onPageSelected = function (position) {
            if (this._indicators != null) {
                if (this._indicators.mSnap || this._indicators.mScrollState == android.support.v4.view.ViewPager.SCROLL_STATE_IDLE) {
                    this._indicators.mCurrentPage = position;
                    this._indicators.mSnapPage = position;
                    this._indicators.invalidate();
                }
            }
            this._owner.position = position;
        };
        PageChangedListenerInner.prototype.onPageScrollStateChanged = function (state) {
            if (this._indicators != null) {
                this._indicators.mScrollState = state;
            }
            if (state == 0) {
                var eventData = {
                    eventName: "positionSelected",
                    object: this._owner
                };
                this._owner.notify(eventData);
            }
        };
        return PageChangedListenerInner;
    }(android.support.v4.view.ViewPager.SimpleOnPageChangeListener));
    PageChangedListenerClass = PageChangedListenerInner;
}
var VerticalViewPagerClass;
function ensureVerticalViewPagerClass() {
    if (VerticalViewPagerClass) {
        return;
    }
    var VerticalViewPagerInner = (function (_super) {
        __extends(VerticalViewPagerInner, _super);
        function VerticalViewPagerInner(context, attrs) {
            var _this = _super.call(this, context, attrs) || this;
            global.__native(_this).setPageTransformer(false, new android.support.v4.view.ViewPager.PageTransformer({
                transformPage: function (page, position) {
                    page.setTranslationX(page.getWidth() * -position);
                    var yPosition = position * page.getHeight();
                    page.setTranslationY(yPosition);
                }
            }));
            // get rid of the overscroll drawing that happens on the left and right
            global.__native(_this).setOverScrollMode(android.view.View.OVER_SCROLL_NEVER);
            return global.__native(_this);
        }
        VerticalViewPagerInner.prototype.swapTouchEvent = function (ev) {
            var width = this.getWidth();
            var height = this.getHeight();
            var swappedX = (ev.getY() / height) * width;
            var swappedY = (ev.getX() / width) * height;
            ev.setLocation(swappedX, swappedY);
            return ev;
        };
        VerticalViewPagerInner.prototype.onInterceptTouchEvent = function (ev) {
            var intercept = _super.prototype.onInterceptTouchEvent.call(this, this.swapTouchEvent(ev));
            //If not intercept, touch event should not be swapped.
            this.swapTouchEvent(ev);
            return intercept;
        };
        VerticalViewPagerInner.prototype.onTouchEvent = function (ev) {
            return _super.prototype.onTouchEvent.call(this, this.swapTouchEvent(ev));
        };
        return VerticalViewPagerInner;
    }(android.support.v4.view.ViewPager));
    VerticalViewPagerClass = VerticalViewPagerInner;
}
var CirclePageIndicatorClass;
function ensureCirclePageIndicatorClass() {
    if (CirclePageIndicatorClass) {
        return;
    }
    var CirclePageIndicatorInner = (function (_super) {
        __extends(CirclePageIndicatorInner, _super);
        function CirclePageIndicatorInner(context) {
            var _this = _super.call(this, context, null) || this;
            _this.HORIZONTAL = 0;
            _this.VERTICAL = 1;
            _this.mCentered = true;
            _this.mPaintFill = new android.graphics.Paint(android.graphics.Paint.ANTI_ALIAS_FLAG);
            _this.mPaintFill.setStyle(android.graphics.Paint.Style.FILL);
            _this.mPaintFill.setColor(android.graphics.Color.parseColor("#808080"));
            _this.mPaintPageFill = new android.graphics.Paint(android.graphics.Paint.ANTI_ALIAS_FLAG);
            _this.mPaintPageFill.setStyle(android.graphics.Paint.Style.FILL);
            _this.mPaintPageFill.setColor(android.graphics.Color.parseColor("#c0c0c0"));
            _this.mOrientation = _this.HORIZONTAL;
            _this.mRadius = _this.dpToPx(3);
            _this.mSnap = true;
            _this.mPaintStroke = new android.graphics.Paint(android.graphics.Paint.ANTI_ALIAS_FLAG);
            _this.mPaintStroke.setStyle(android.graphics.Paint.Style.STROKE);
            _this.mPaintStroke.setColor(android.graphics.Color.parseColor("#FFFFFFFF"));
            _this.mPaintStroke.setStrokeWidth(0);
            var p = _this.dpToPx(14);
            global.__native(_this).setPadding(p, p, p, p);
            global.__native(_this).setBackgroundColor(android.graphics.Color.TRANSPARENT);
            return global.__native(_this);
        }
        CirclePageIndicatorInner.prototype.SetViewPager = function (view, initialPosition) {
            if (view.getAdapter() == null) {
                throw new java.lang.IllegalStateException("ViewPager does not have adapter instance.");
            }
            this.mViewPager = view;
            this.UpdatePageSize();
            this.SetCurrentItem(initialPosition);
        };
        CirclePageIndicatorInner.prototype.SetCurrentItem = function (item) {
            if (this.mViewPager == null) {
                throw new java.lang.IllegalStateException("ViewPager has not been bound.");
            }
            this.mViewPager.setCurrentItem(item);
            this.mCurrentPage = item;
            this.invalidate();
        };
        CirclePageIndicatorInner.prototype.UpdatePageSize = function () {
            if (this.mViewPager != null) {
                this.mPageSize = (this.mOrientation == this.HORIZONTAL) ? this.mViewPager.getWidth() : this.mViewPager.getHeight();
            }
        };
        CirclePageIndicatorInner.prototype.dpToPx = function (dp) {
            var res = android.content.res.Resources;
            return dp * res.getSystem().getDisplayMetrics().density;
        };
        CirclePageIndicatorInner.prototype.SetPageColor = function (pageColor) {
            this.mPaintPageFill.setColor(Number(pageColor));
            this.invalidate();
        };
        CirclePageIndicatorInner.prototype.SetFillColor = function (fillColor) {
            this.mPaintFill.setColor(Number(fillColor));
            this.invalidate();
        };
        CirclePageIndicatorInner.prototype.onDraw = function (canvas) {
            _super.prototype.onDraw.call(this, canvas);
            if (this.mViewPager == null) {
                return;
            }
            var count = this.mViewPager.getAdapter().getCount();
            if (count == 0) {
                return;
            }
            if (this.mCurrentPage >= count) {
                this.SetCurrentItem(count - 1);
                return;
            }
            var longSize;
            var longPaddingBefore;
            var longPaddingAfter;
            var shortPaddingBefore;
            if (this.mOrientation == this.HORIZONTAL) {
                longSize = this.getWidth();
                longPaddingBefore = this.getPaddingLeft();
                longPaddingAfter = this.getPaddingRight();
                shortPaddingBefore = this.getPaddingTop();
            }
            else {
                longSize = this.getHeight();
                longPaddingBefore = this.getPaddingTop();
                longPaddingAfter = this.getPaddingBottom();
                shortPaddingBefore = this.getPaddingLeft();
            }
            var threeRadius = this.mRadius * 4; // indicators separation
            var shortOffset = shortPaddingBefore + this.mRadius;
            var longOffset = longPaddingBefore + this.mRadius;
            if (this.mCentered) {
                longOffset += ((longSize - longPaddingBefore - longPaddingAfter) / 2.0) - ((count * threeRadius) / 2.0);
            }
            var dX;
            var dY;
            var pageFillRadius = this.mRadius;
            if (this.mPaintStroke.getStrokeWidth() > 0) {
                pageFillRadius -= this.mPaintStroke.getStrokeWidth() / 2.0;
            }
            //Draw stroked circles
            for (var iLoop = 0; iLoop < count; iLoop++) {
                var drawLong = longOffset + (iLoop * threeRadius);
                if (this.mOrientation == this.HORIZONTAL) {
                    dX = drawLong;
                    dY = shortOffset;
                }
                else {
                    dX = shortOffset;
                    dY = drawLong;
                }
                // Only paint fill if not completely transparent
                if (this.mPaintPageFill.getAlpha() > 0) {
                    canvas.drawCircle(dX, dY, pageFillRadius, this.mPaintPageFill);
                }
                // Only paint stroke if a stroke width was non-zero
                if (pageFillRadius != this.mRadius) {
                    canvas.drawCircle(dX, dY, this.mRadius, this.mPaintStroke);
                }
            }
            //Draw the filled circle according to the current scroll
            var cx = (this.mSnap ? this.mSnapPage : this.mCurrentPage) * threeRadius;
            if (!this.mSnap && (this.mPageSize != 0)) {
                cx += (this.mCurrentOffset * 1.0 / this.mPageSize) * threeRadius;
            }
            if (this.mOrientation == this.HORIZONTAL) {
                dX = longOffset + cx;
                dY = shortOffset;
            }
            else {
                dX = shortOffset;
                dY = longOffset + cx;
            }
            canvas.drawCircle(dX, dY, this.mRadius, this.mPaintFill);
        };
        CirclePageIndicatorInner.prototype.onMeasure = function (widthMeasureSpec, heightMeasureSpec) {
            if (this.mOrientation == this.HORIZONTAL) {
                this.setMeasuredDimension(this.MeasureLong(widthMeasureSpec), this.MeasureShort(heightMeasureSpec));
            }
            else {
                this.setMeasuredDimension(this.MeasureShort(widthMeasureSpec), this.MeasureLong(heightMeasureSpec));
            }
        };
        CirclePageIndicatorInner.prototype.MeasureLong = function (measureSpec) {
            var result = 0;
            var specMode = android.view.View.MeasureSpec.getMode(measureSpec);
            var specSize = android.view.View.MeasureSpec.getSize(measureSpec);
            if ((specMode == android.view.View.MeasureSpec.EXACTLY) || (this.mViewPager == null) || (this.mViewPager.getAdapter() == null)) {
                //We were told how big to be
                result = specSize;
            }
            else {
                //Calculate the width according the views count
                var count = this.mViewPager.getAdapter().getCount();
                result = (this.getPaddingLeft() + this.getPaddingRight()
                    + (count * 2 * this.mRadius) + (count - 1) * this.mRadius + 1);
                //Respect AT_MOST value if that was what is called for by measureSpec
                if (specMode == android.view.View.MeasureSpec.AT_MOST) {
                    result = java.lang.Math.min(result, specSize);
                }
            }
            return result;
        };
        CirclePageIndicatorInner.prototype.MeasureShort = function (measureSpec) {
            var result = 0;
            var specMode = android.view.View.MeasureSpec.getMode(measureSpec);
            var specSize = android.view.View.MeasureSpec.getSize(measureSpec);
            if (specMode == android.view.View.MeasureSpec.EXACTLY) {
                //We were told how big to be
                result = specSize;
            }
            else {
                //Measure the height
                result = (2 * this.mRadius + this.getPaddingTop() + this.getPaddingBottom() + 1);
                //Respect AT_MOST value if that was what is called for by measureSpec
                if (specMode == android.view.View.MeasureSpec.AT_MOST) {
                    result = java.lang.Math.min(result, specSize);
                }
            }
            return result;
        };
        CirclePageIndicatorInner.prototype.onRestoreInstanceState = function (state) {
            try {
                var bundle = state;
                if (bundle != null) {
                    var superState = bundle.getParcelable("base");
                    if (superState != null)
                        _super.prototype.onRestoreInstanceState.call(this, superState);
                    this.mCurrentPage = bundle.getInt("mCurrentPage", 0);
                    this.mSnapPage = bundle.getInt("mCurrentPage", 0);
                }
            }
            catch (e) {
                _super.prototype.onRestoreInstanceState.call(this, state);
            }
            this.requestLayout();
        };
        CirclePageIndicatorInner.prototype.onSaveInstanceState = function () {
            var superState = _super.prototype.onSaveInstanceState.call(this);
            var state = new android.os.Bundle();
            state.putParcelable("base", superState);
            state.putInt("mCurrentPage", this.mCurrentPage);
            return state;
        };
        return CirclePageIndicatorInner;
    }(android.view.View));
    CirclePageIndicatorClass = CirclePageIndicatorInner;
}
//# sourceMappingURL=carousel-view.android.js.map