"use strict";
var common = require("./carousel-view-common");
var colorModule = require("color");
var CarouselView = (function (_super) {
    __extends(CarouselView, _super);
    function CarouselView() {
        var _this = _super.call(this) || this;
        // As custom properties returns default value only here
        // Adding a container so we can build UIPageViewController onLoaded()
        _this._ios = new UIView();
        return _this;
    }
    Object.defineProperty(CarouselView.prototype, "ios", {
        get: function () {
            return this._ios;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CarouselView.prototype, "_nativeView", {
        // Thanks to NathanWalker for the _nativeView tip
        get: function () {
            return this._ios;
        },
        enumerable: true,
        configurable: true
    });
    // Thanks to NathanWalker for the onLoaded tip
    CarouselView.prototype.onLoaded = function () {
        var orientation;
        if (this.orientation == 0)
            orientation = UIPageViewControllerNavigationOrientation.UIPageViewControllerNavigationOrientationHorizontal;
        else
            orientation = UIPageViewControllerNavigationOrientation.UIPageViewControllerNavigationOrientationVertical;
        var objects = [UIPageViewControllerSpineLocation.UIPageViewControllerSpineLocationNone, this.interPageSpacing];
        var keys = [UIPageViewControllerOptionSpineLocationKey, UIPageViewControllerOptionInterPageSpacingKey];
        this._pageController = UIPageViewController.alloc().initWithTransitionStyleNavigationOrientationOptions(UIPageViewControllerTransitionStyle.UIPageViewControllerTransitionStyleScroll, orientation, NSDictionary.dictionaryWithObjectsForKeys(objects, keys));
        // TODO: Create a property for the user set the margin color (background color)
        var that = new WeakRef(this);
        this._pageController.dataSource = DataSourceClass.initWithOwner(that);
        this._pageController.delegate = DelegateClass.initWithOwner(that);
        var firstViewController = this.createViewController(this.position);
        var direction = UIPageViewControllerNavigationDirection.UIPageViewControllerNavigationDirectionForward;
        this._pageController.setViewControllersDirectionAnimatedCompletion([firstViewController], direction, false, function (arg1) { });
        this._ios.addSubview(this._pageController.view);
        if (this.showIndicators) {
            this._pageControl = new UIPageControl();
            this._pageControl.pageIndicatorTintColor = new colorModule.Color("#c0c0c0").ios;
            this._pageControl.currentPageIndicatorTintColor = new colorModule.Color("#808080").ios;
            this._pageControl.translatesAutoresizingMaskIntoConstraints = false;
            this._pageControl.enabled = false;
            this.ConfigurePageControl();
            this._ios.addSubview(this._pageControl);
            var viewsDictionary = NSDictionary.dictionaryWithObjectForKey(this._pageControl, "pageControl");
            if (this.orientation == 0) {
                this._ios.addConstraints(NSLayoutConstraint.constraintsWithVisualFormatOptionsMetricsViews("H:|-[pageControl]-|", NSLayoutFormatOptions.NSLayoutFormatAlignAllCenterX, null, viewsDictionary));
                this._ios.addConstraints(NSLayoutConstraint.constraintsWithVisualFormatOptionsMetricsViews("V:[pageControl]|", 0, null, viewsDictionary));
            }
            else {
                this._pageControl.transform = CGAffineTransformMakeRotation(3.14159265 / 2);
                this._ios.addConstraints(NSLayoutConstraint.constraintsWithVisualFormatOptionsMetricsViews("[pageControl(==36)]", 0, null, viewsDictionary));
                this._ios.addConstraints(NSLayoutConstraint.constraintsWithVisualFormatOptionsMetricsViews("H:[pageControl]|", 0, null, viewsDictionary));
                this._ios.addConstraints(NSLayoutConstraint.constraintsWithVisualFormatOptionsMetricsViews("V:|-[pageControl]-|", NSLayoutFormatOptions.NSLayoutFormatAlignAllCenterY, null, viewsDictionary));
            }
        }
        var eventData = {
            eventName: "positionSelected",
            object: this
        };
        this.notify(eventData);
    };
    CarouselView.prototype.ConfigurePageControl = function () {
        if (this._pageControl != null && this.itemsSource != null) {
            this._pageControl.numberOfPages = this.itemsSource.length;
            this._pageControl.currentPage = this.position;
        }
    };
    CarouselView.prototype.insertPage = function (position, bindingContext) {
        return __awaiter(this, void 0, void 0, function () {
            var firstViewController, direction;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this._pageController != null))
                            return [3 /*break*/, 2];
                        if (position > this.itemsSource.length)
                            throw new Error("Index out of bounds (position > itemsSource length).");
                        if (position < 0)
                            throw new Error("Index out of bounds (position < 0).");
                        if (position == this.itemsSource.length)
                            this.itemsSource.push(bindingContext);
                        else
                            this.itemsSource.splice(position, 0, bindingContext);
                        firstViewController = this._pageController.viewControllers[0];
                        direction = UIPageViewControllerNavigationDirection.UIPageViewControllerNavigationDirectionForward;
                        // Using a standard JS Array here (just type-cast to any to suffice TypeScript)
                        // {N} will auto-marshall this into a NSArray when making the call since the metadata knows its
                        // supposed to be an NSArray :)
                        // Thanks to NathanWalker for the auto-marshall tip for NSArray
                        this._pageController.setViewControllersDirectionAnimatedCompletion([firstViewController], direction, false, function (arg1) { });
                        this.ConfigurePageControl();
                        return [4 /*yield*/, this.delay(100)];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    CarouselView.prototype.removePage = function (position) {
        return __awaiter(this, void 0, void 0, function () {
            var newPos, forward, reverse, direction, firstViewController, firstViewController, direction, eventData;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this._pageController != null))
                            return [3 /*break*/, 4];
                        if (position > this.itemsSource.length - 1)
                            throw new Error("Index out of bounds (position > itemsSource length - 1).");
                        if (position < 0)
                            throw new Error("Index out of bounds (position < 0).");
                        this.itemsSource.splice(position, 1);
                        if (!(position == this.position))
                            return [3 /*break*/, 2];
                        newPos = position - 1;
                        if (newPos == -1)
                            newPos = 0;
                        return [4 /*yield*/, this.delay(100)];
                    case 1:
                        _a.sent();
                        forward = UIPageViewControllerNavigationDirection.UIPageViewControllerNavigationDirectionForward;
                        reverse = UIPageViewControllerNavigationDirection.UIPageViewControllerNavigationDirectionReverse;
                        direction = position == 0 ? forward : reverse;
                        firstViewController = this.createViewController(newPos);
                        this._pageController.setViewControllersDirectionAnimatedCompletion([firstViewController], direction, true, function (arg1) { });
                        this.position = newPos;
                        return [3 /*break*/, 3];
                    case 2:
                        firstViewController = this._pageController.viewControllers[0];
                        direction = UIPageViewControllerNavigationDirection.UIPageViewControllerNavigationDirectionForward;
                        this._pageController.setViewControllersDirectionAnimatedCompletion([firstViewController], direction, false, function (arg1) { });
                        _a.label = 3;
                    case 3:
                        this.ConfigurePageControl();
                        eventData = {
                            eventName: "positionSelected",
                            object: this
                        };
                        this.notify(eventData);
                        _a.label = 4;
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    CarouselView.prototype.setCurrentPage = function (position) {
        if (this._pageController != null) {
            if (position > this.itemsSource.length - 1)
                throw new Error("Index out of bounds (position > itemsSource length - 1).");
            if (position < 0)
                throw new Error("Index out of bounds (position < 0).");
            var forward = UIPageViewControllerNavigationDirection.UIPageViewControllerNavigationDirectionForward;
            var reverse = UIPageViewControllerNavigationDirection.UIPageViewControllerNavigationDirectionReverse;
            var direction = position > this.position ? forward : reverse;
            this.position = position;
            var firstViewController = this.createViewController(position);
            this._pageController.setViewControllersDirectionAnimatedCompletion([firstViewController], direction, true, function (arg1) { });
            this.ConfigurePageControl();
            var eventData = {
                eventName: "positionSelected",
                object: this
            };
            this.notify(eventData);
        }
    };
    CarouselView.prototype.itemsSourceChanged = function () {
        if (this._pageController != null) {
            if (this.position > this.itemsSource.length - 1)
                this.position = this.itemsSource.length - 1;
            var firstViewController = this.createViewController(this.position);
            var direction = UIPageViewControllerNavigationDirection.UIPageViewControllerNavigationDirectionForward;
            this._pageController.setViewControllersDirectionAnimatedCompletion([firstViewController], direction, false, function (arg1) { });
            this.ConfigurePageControl();
            var eventData = {
                eventName: "positionSelected",
                object: this
            };
            this.notify(eventData);
        }
    };
    CarouselView.prototype.createViewController = function (position) {
        var item;
        if (this.itemsSource != null)
            item = this.itemsSource.getItem(position);
        var view = this.templateSelector.OnSelectTemplate(position, item);
        var obj = view;
        this._addView(obj);
        var viewController = new ViewContainer();
        viewController.tag = position;
        viewController.view = obj._view;
        viewController.owner = view;
        obj.onLoaded();
        return viewController;
    };
    CarouselView.prototype.delay = function (ms) {
        return new Promise(function (resolve) { return setTimeout(resolve, ms); });
    };
    return CarouselView;
}(common.CarouselView));
exports.CarouselView = CarouselView;
var DataSourceClass = (function (_super) {
    __extends(DataSourceClass, _super);
    function DataSourceClass() {
        return _super.apply(this, arguments) || this;
    }
    Object.defineProperty(DataSourceClass.prototype, "owner", {
        get: function () {
            return this._owner.get();
        },
        enumerable: true,
        configurable: true
    });
    DataSourceClass.initWithOwner = function (owner) {
        var datasource = new DataSourceClass();
        datasource._owner = owner;
        return datasource;
    };
    DataSourceClass.prototype.pageViewControllerViewControllerBeforeViewController = function (pageViewController, viewController) {
        var controller = viewController;
        var position = Number(controller.tag);
        // Determine if we are on the first page
        if (position == 0) {
            // We are on the first page, so there is no need for a controller before that
            return null;
        }
        else {
            var previousPageIndex = position - 1;
            return this.owner.createViewController(previousPageIndex);
        }
    };
    DataSourceClass.prototype.pageViewControllerViewControllerAfterViewController = function (pageViewController, viewController) {
        var controller = viewController;
        var position = Number(controller.tag);
        // Determine if we are on the last page
        var count = this.presentationCountForPageViewController(pageViewController);
        if (position == count - 1) {
            // We are on the last page, so there is no need for a controller after that
            return null;
        }
        else {
            var nextPageIndex = position + 1;
            return this.owner.createViewController(nextPageIndex);
        }
    };
    DataSourceClass.prototype.presentationCountForPageViewController = function (pageViewController) {
        // FIX: populate the carousel with data after being loaded in UI
        if (this.owner.itemsSource == null)
            return 0;
        return this.owner.itemsSource.length;
    };
    return DataSourceClass;
}(NSObject));
DataSourceClass.ObjCProtocols = [UIPageViewControllerDataSource];
var DelegateClass = (function (_super) {
    __extends(DelegateClass, _super);
    function DelegateClass() {
        return _super.apply(this, arguments) || this;
    }
    Object.defineProperty(DelegateClass.prototype, "owner", {
        get: function () {
            return this._owner.get();
        },
        enumerable: true,
        configurable: true
    });
    DelegateClass.initWithOwner = function (owner) {
        var delegate = new DelegateClass();
        delegate._owner = owner;
        return delegate;
    };
    DelegateClass.prototype.pageViewControllerDidFinishAnimatingPreviousViewControllersTransitionCompleted = function (pageViewController, finished, previousViewControllers, completed) {
        if (finished) {
            var controller = pageViewController.viewControllers[0];
            var position = controller.tag;
            this.owner.position = position;
            this.owner.ConfigurePageControl();
            var eventData = {
                eventName: "positionSelected",
                object: this.owner
            };
            this.owner.notify(eventData);
        }
    };
    return DelegateClass;
}(NSObject));
DelegateClass.ObjCProtocols = [UIPageViewControllerDelegate];
var ViewContainer = (function (_super) {
    __extends(ViewContainer, _super);
    function ViewContainer() {
        return _super.apply(this, arguments) || this;
    }
    // Thanks to tzraikov
    ViewContainer.prototype.viewDidLoad = function () {
        _super.prototype.viewDidLoad.call(this);
        if (this.owner) {
            this.owner.onLoaded();
        }
    };
    // Thanks to tzraikov
    ViewContainer.prototype.viewDidLayoutSubviews = function () {
        if (this.owner) {
            var width = this.view.frame.size.width;
            var height = this.view.frame.size.height;
            this.owner.measure(width, height);
            this.owner.layout(0, 0, width, height);
            this.owner._updateLayout();
        }
    };
    return ViewContainer;
}(UIViewController));
exports.ViewContainer = ViewContainer;
//# sourceMappingURL=carousel-view.ios.js.map